@@ -1,6 +1,6 @@
 /* otftotfm.cc -- driver for translating OpenType fonts to TeX metrics
  *
- * Copyright (c) 2003-2011 Eddie Kohler
+ * Copyright (c) 2003-2012 Eddie Kohler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -50,6 +50,10 @@
 #include <signal.h>
 #include <algorithm>
 #include <math.h>
+/* M_PI isn't defined in C99 */
+#ifndef M_PI
+# define M_PI 3.14159265358979323846264338327
+#endif
 #ifdef HAVE_CTIME
 # include <time.h>
 #endif
@@ -1307,18 +1311,37 @@ main_dvips_map(const String &ps_name, const FontInfo &finfo, ErrorHandler *errh)
 }
 
 static void
-do_gsub(Metrics& metrics, const OpenType::Font& otf, DvipsEncoding& dvipsenc, bool dvipsenc_literal, HashMap<uint32_t, int>& feature_usage, const Vector<PermString>& glyph_names, ErrorHandler* errh)
+do_gsub(Metrics& metrics, const OpenType::Font& otf,
+	DvipsEncoding& dvipsenc, bool dvipsenc_literal,
+	HashMap<uint32_t, int>& feature_usage,
+	const Vector<PermString>& glyph_names, ErrorHandler* errh)
 {
-    // apply activated GSUB features
+    // find activated GSUB features
     OpenType::Gsub gsub(otf.table("GSUB"), &otf, errh);
     Vector<Lookup> lookups(gsub.nlookups(), Lookup());
     find_lookups(gsub.script_list(), gsub.feature_list(), lookups, errh);
+
+    // find all characters that might result
+    Vector<bool> used(glyph_names.size(), false);
+    for (Metrics::Code c = 0; c < metrics.encoding_size(); ++c) {
+	Metrics::Glyph g = metrics.glyph(c);
+	if (g >= 0 && g < used.size())
+	    used[g] = true;
+    }
+    for (int i = 0; i < lookups.size(); ++i)
+	if (lookups[i].used) {
+	    OpenType::GsubLookup l = gsub.lookup(i);
+	    l.mark_out_glyphs(gsub, used);
+	}
+    OpenType::Coverage used_coverage(used);
+
+    // apply activated GSUB features
     Vector<OpenType::Substitution> subs;
     for (int i = 0; i < lookups.size(); i++)
 	if (lookups[i].used) {
 	    OpenType::GsubLookup l = gsub.lookup(i);
 	    subs.clear();
-	    bool understood = l.unparse_automatics(gsub, subs);
+	    bool understood = l.unparse_automatics(gsub, subs, used_coverage);
 
 	    // check for -ffina, which should apply only at the ends of words,
 	    // and -finit, which should apply only at the beginnings.
@@ -1360,7 +1383,7 @@ do_gsub(Metrics& metrics, const OpenType::Font& otf, DvipsEncoding& dvipsenc, bo
 	    if (alt_lookups[i].used) {
 		OpenType::GsubLookup l = gsub.lookup(i);
 		alt_subs.clear();
-		(void) l.unparse_automatics(gsub, alt_subs);
+		(void) l.unparse_automatics(gsub, alt_subs, used_coverage);
 		metrics.apply_alternates(alt_subs, i, *alt_lookups[i].filter, glyph_names);
 	    }
 	altselector_features.swap(interesting_features);
@@ -2094,7 +2117,7 @@ main(int argc, char *argv[])
 
 	  case VERSION_OPT:
 	    printf("otftotfm (LCDF typetools) %s\n", VERSION);
-	    printf("Copyright (C) 2002-2011 Eddie Kohler\n\
+	    printf("Copyright (C) 2002-2012 Eddie Kohler\n\
 This is free software; see the source for copying conditions.\n\
 There is NO warranty, not even for merchantability or fitness for a\n\
 particular purpose.\n");
@@ -2184,7 +2207,7 @@ particular purpose.\n");
 		    errh->message("glyphlist.txt found with kpathsea at %s", g.c_str());
 	    } else
 #endif
-		glyphlist_files.push_back(SHAREDIR "/glyphlist.txt");
+		glyphlist_files.push_back(GLYPHLISTDIR "/glyphlist.txt");
 #if HAVE_KPATHSEA
 	    if (String g = kpsei_find_file("texglyphlist.txt", KPSEI_FMT_MAP)) {
 		glyphlist_files.push_back(g);
@@ -2192,7 +2215,7 @@ particular purpose.\n");
 		    errh->message("texglyphlist.txt found with kpathsea at %s", g.c_str());
 	    } else
 #endif
-		glyphlist_files.push_back(SHAREDIR "/texglyphlist.txt");
+		glyphlist_files.push_back(GLYPHLISTDIR "/texglyphlist.txt");
 	}
 
 	// read glyphlist
