@@ -1,6 +1,6 @@
 /* t1rawafm.cc -- driver for generating a raw AFM file from a font
  *
- * Copyright (c) 2008-2011 Eddie Kohler
+ * Copyright (c) 2008-2012 Eddie Kohler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -64,7 +64,7 @@ usage_error(ErrorHandler *errh, const char *error_message, ...)
     va_list val;
     va_start(val, error_message);
     if (!error_message)
-	errh->message("Usage: %s [OPTION]... FONT", program_name);
+	errh->message("Usage: %s [OPTION]... [FONT [OUTPUT]]", program_name);
     else
 	errh->vxmessage(ErrorHandler::e_error, error_message, val);
     errh->message("Type %s --help for more information.", program_name);
@@ -79,13 +79,13 @@ usage()
 %<T1rawafm%> generates a raw (kernless and ligatureless) AFM file corresponding\n\
 to the specified Type 1 font file and writes it to the standard output.\n\
 \n\
-Usage: %s [OPTION]... [FONT]\n\
+Usage: %s [OPTION]... [FONT [OUTPUT]]\n\
 \n\
-FONT is the name of a PFA or PFB font file. If omitted, t1testpage will read\n\
-a font file from the standard input.\n\
+FONT is the name of a PFA or PFB font file. If omitted, t1rawafm will read a\n\
+font file from the standard input.\n\
 \n\
 Options:\n\
-  -o, --output=FILE            Write output to FILE instead of standard out.\n\
+  -o, --output=FILE            Write output to FILE instead of standard output.\n\
   -h, --help                   Print this message and exit.\n\
       --version                Print version number and exit.\n\
 \n\
@@ -299,7 +299,8 @@ main(int argc, char *argv[])
 	int opt = Clp_Next(clp);
 	switch (opt) {
 
-	  case OUTPUT_OPT:
+	case OUTPUT_OPT:
+	output_file:
 	    if (output_file)
 		errh->fatal("output file already specified");
 	    output_file = clp->vstr;
@@ -307,7 +308,7 @@ main(int argc, char *argv[])
 
 	  case VERSION_OPT:
 	    printf("t1rawafm (LCDF typetools) %s\n", VERSION);
-	    printf("Copyright (C) 2008-2011 Eddie Kohler\n\
+	    printf("Copyright (C) 2008-2012 Eddie Kohler\n\
 This is free software; see the source for copying conditions.\n\
 There is NO warranty, not even for merchantability or fitness for a\n\
 particular purpose.\n");
@@ -321,8 +322,9 @@ particular purpose.\n");
 
 	  case Clp_NotOption:
 	    if (font)
-		errh->fatal("font already specified");
-	    do_file(clp->vstr, psres, errh);
+		goto output_file;
+	    else
+		do_file(clp->vstr, psres, errh);
 	    break;
 
 	  case Clp_Done:
