@@ -1,6 +1,6 @@
 /* t1reencode.cc -- driver for reencoding Type 1 fonts
  *
- * Copyright (c) 2005-2010 Eddie Kohler
+ * Copyright (c) 2005-2012 Eddie Kohler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -23,7 +23,7 @@
 #include <ctype.h>
 #include <errno.h>
 #include "util.hh"
-#include "md5.h"
+#include <lcdf/md5.h>
 #ifdef HAVE_CTIME
 # include <time.h>
 #endif
@@ -48,7 +48,7 @@ const Clp_Option options[] = {
     { "help", 'h', HELP_OPT, 0, 0 },
     { "output", 'o', OUTPUT_OPT, Clp_ValString, 0 },
     { "pfa", 'a', PFA_OPT, 0, 0 },
-    { "pfb", 'b', PFA_OPT, 0, 0 },
+    { "pfb", 'b', PFB_OPT, 0, 0 },
     { "name", 'n', FONTNAME_OPT, Clp_ValString, 0 },
     { "fullname", 'N', FULLNAME_OPT, Clp_ValString, 0 },
     { "full-name", 'N', FULLNAME_OPT, Clp_ValString, 0 },
@@ -978,7 +978,7 @@ main(int argc, char *argv[])
 
 	  case VERSION_OPT:
 	    printf("t1reencode (LCDF typetools) %s\n", VERSION);
-	    printf("Copyright (C) 1999-2010 Eddie Kohler\n\
+	    printf("Copyright (C) 1999-2012 Eddie Kohler\n\
 This is free software; see the source for copying conditions.\n\
 There is NO warranty, not even for merchantability or fitness for a\n\
 particular purpose.\n");
